# ESP8266 Micropython Installation
_Disclaimer:_ This is the method I have used to install/connect to the equipment I have listed below **using a Mac**. 
It is hoped that these steps will work on your hardware but differences in manufacturer, version, storage, firmware, RAM, etc. may change your results.
**Your mileage may (and probably will) vary.**
## Hardware
* [HiLetgo ESP8266 NodeMCU LUA CP2102 ESP-12E](http://a.co/8UUxd50)
* [Adafruit Feather Huzzah with Wifi](https://www.adafruit.com/product/2821)

## Software Installed on Mac

##### [ampy](https://github.com/adafruit/ampy)
Utility to interact with a MicroPython board over a serial connection
```bash
pip install adafruit-ampy
```

##### [esptool](https://github.com/espressif/esptool)
A serial bootloader utility.
```bash
pip install esptool
```

##### [Mac_OSX_VCP_Driver](https://www.silabs.com/documents/public/software/Mac_OSX_VCP_Driver.zip)
CP210x USB to UART Bridge VCP Drivers.

Download the MacOSX driver and follow the install instructions.

##### [esp8266-20171101-v1.9.3.bin](http://micropython.org/download#esp8266)
Stable MicroPython firmware for ESP8266. _Obviously this version will change over time._

## Installing MicroPython

#### Connect hardware
Using a (seemingly standard) Micro USB cable, connect your Mac to the micro USB port on the ESP8285.

_Note: I have heard some charging only cables aren't wired to pass data, so make sure the cable specifies data sync as well._

The blue LED on the ESP8266 should blink VERY briefly when powered up.

#### Erase the existing flash
Referencing your particular serial port mentioned above:
```bash
esptool.py --port /dev/SLAB_USBtoUART erase_flash
```

#### Install MicroPython firmware
Referencing your particular serial port and MicroPython version mentioned above:
```bash
esptool.py --port /dev/SLAB_USBtoUART write_flash -fm dio 0x00000 esp8266-20171101-v1.9.3.bin
```

_Please note the options for baud, fm, and fs.
These seemed to differ from board to board, manufacturer, etc. 
These values worked for me for the hardware above (after some experimentation)._

The install indicates that the board is reset afterwards BUT I unplugged and replugged just to make sure.

## Getting a MicroPython REPL prompt
There are a variety of serial terminal packages available, I use **screen**. 
These instruction are for using screen, but you get the idea
```bash
brew install screen
```
Make the connection:
```bash
screen /dev/SLAB_USBtoUART 115200
```
_As stated above, your device name may vary._

Hopefully, something similar to the following will be displayed:
```bash
MicroPython v1.9.3-8-g63826ac5c on 2017-11-01; ESP module with ESP8266
Type "help()" for more information.
>>>
```

If NOT, play around with the baud rates (using other values than 115200 mentioned above).
