# ESP8285 Micropython Installation
_Disclaimer:_ This is the method I have used to install/connect to the equipment I have listed below **using a Mac**. 
It is hoped that these steps will work on your hardware but differences in manufacturer, version, storage, firmware, RAM, etc. may change your results.
**Your mileage may (and probably will) vary.**
## Hardware
* [Wemos D1 Mini Development Board ESP8285 (V1.0.0 1MB Flash Lite Wireless WiFi)](https://www.amazon.com/gp/product/B075H8W8NK)

## Software Installed on Mac

##### [ampy](https://github.com/adafruit/ampy)
Utility to interact with a MicroPython board over a serial connection
```bash
pip install adafruit-ampy
```

##### [esptool](https://github.com/espressif/esptool)
A serial bootloader utility.
```bash
pip install esptool
```

##### [CH341SER_MAC.ZIP](http://www.wch.cn/downfile/178)
CH340 / CH341 USB to serial installation package MAC OS drivers, support 32/64 Bit system with instructions for use.

_Note: I felt REALLY nervous about downloading and installing this as the site out of China as I don't speak Chinese.
Please execute your own caution.
Google translate did a fairly good job of deciphering the site for me.
The drivers seem to be working great!_

Download, unzip, and install.

This **should** create a device in your Mac's /dev directory:
```bash
ls /dev/cu.wchusbserial*
```
should respond with something similar to this:
```bash
/dev/cu.wchusbserial40110
```
This is our serial device that we will use to connect the ESP8285. **Make note of this.**

##### [esp8266-20171101-v1.9.3.bin](http://micropython.org/download#esp8266)
Stable MicroPython firmware for ESP8266. _Obviously this version will change over time._

## Installing MicroPython

#### Connect hardware
Using a (seemingly standard) Micro USB cable, connect your Mac to the micro USB port on the ESP8285.

_Note: I have heard some charging only cables aren't wired to pass data, so make sure the cable specifies data sync as well._

The blue LED on the ESP8285 should blink VERY briefly when powered up.

#### Erase the existing flash
Referencing your particular serial port mentioned above:
```bash
esptool.py --port /dev/cu.wchusbserial40110 erase_flash
```

#### Install MicroPython firmware
Referencing your particular serial port and MicroPython version mentioned above:
```bash
esptool.py --port /dev/cu.wchusbserial40110 --baud 115200 write_flash -fm dout -fs 8m 0x00000 esp8266-20171101-v1.9.3.bin
```

_Please note the options for baud, fm, and fs.
These seemed to differ from board to board, manufacturer, etc. 
These values worked for me for the hardware above (after some experimentation)._

The install indicates that the board is reset afterwards BUT I unplugged and replugged just to make sure.

## Getting a MicroPython REPL prompt
There are a variety of serial terminal packages available, I use **screen**. 
These instruction are for using screen, but you get the idea
```bash
brew install screen
```
Make the connection:
```bash
screen /dev/cu.wchusbserial40110 115200
```
_As stated above, your device name may vary._

It is VERY likely that you session will display a bunch of garbage after connecting. 
BUT hopefully, something similar to the following will be displayed as well:
```bash
MicroPython v1.9.3-8-g63826ac5c on 2017-11-01; ESP module with ESP8266
Type "help()" for more information.
>>>
```

If NOT, play around with the baud rates (using other values than 115200 mentioned above).

## Special Note: using ampy with the ESP8285
Using [ampy](https://github.com/adafruit/ampy) to manipulate and maintain files on your ESP8285 makes things much easier.

**However**, please add the **--delay** option with ampy.
It adds some delay into the connection to adjust for the garbage displayed initially.

For instance:
```bash
ampy --delay 2 --port /dev/wchusbserial40110 ls
```
